#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <math.h>

#define GL_GLEXT_PROTOTYPES
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#include "cybunit.h"
#include "kintro2.h"

#define FRAGMENT	"./test4.fp"
#define VECTOR		"./test4.vp"

SDL_Surface *screen;
void *fragment, *vector;
GLuint texture;
int vpId, fpId, program;
float light[3] = {16.0, 32.0, -38.0};

void cleanup(void)
{
	/* clean up */
	printf("vec3 light = vec3(%f,%f,%f);\n",
			light[0],light[1],light[2]);
	glUseProgram(0);
	glDeleteTextures(1, &texture);
	free(fragment);
	free(vector);
	glDeleteShader(vpId);
	glDeleteShader(fpId);
	glDeleteProgram(program);
	SDL_FreeSurface(screen);
	SDL_Quit();
	exit(1);
}

void nice_error(char *info)
{
	printf("Something went wrong.\nAborting. Here is the reason:\n%s",
	       info);
	cleanup();
}

void compile_program(int *prog, const char *vp, const char *fp)
{
	int result;
	char info[1536];

	*prog = glCreateProgram();
	vpId = glCreateShader(GL_VERTEX_SHADER);
	fpId = glCreateShader(GL_FRAGMENT_SHADER);

	glShaderSource(vpId, 1, &vp, 0);
	glShaderSource(fpId, 1, &fp, 0);
	glCompileShader(vpId);
	glCompileShader(fpId);
	glAttachShader(*prog, fpId);
	glAttachShader(*prog, vpId);
	glLinkProgram(*prog);

	glGetObjectParameterivARB(vpId, GL_OBJECT_COMPILE_STATUS_ARB, &result);
	glGetInfoLogARB(vpId, 1024, NULL, (char *)info);
	if (!result)
		nice_error(info);
	if(strlen(info)!=0)
		printf("%s",info);
	
	glGetObjectParameterivARB(fpId, GL_OBJECT_COMPILE_STATUS_ARB, &result);
	glGetInfoLogARB(fpId, 1024, NULL, (char *)info);
	if (!result)
		nice_error(info);
	if(strlen(info)!=0)
		printf("%s",info);
	
	glGetObjectParameterivARB(*prog, GL_OBJECT_LINK_STATUS_ARB, &result);
	glGetInfoLogARB(*prog, 1024, NULL, (char *)info);
	if (!result)
		nice_error(info);
	else
		printf("Link status:\n%s", info);

}

int do_events(void)
{
	int temp;
	SDL_Event event;

	temp = 0;

	if (SDL_PollEvent(&event) >= 0)
		switch (event.type) {
		case SDL_QUIT:
			temp = 1;
			break;
		case SDL_KEYDOWN:
			if ((event.key.keysym.sym == SDLK_ESCAPE) ||
			    (event.key.keysym.sym == SDLK_SPACE))
				temp = 1;
			else if (event.key.keysym.sym == SDLK_INSERT)
				light[0]+=0.35;
			else if (event.key.keysym.sym == SDLK_DELETE)
				light[0]-=0.35;
			else if (event.key.keysym.sym == SDLK_HOME)
				light[1]+=0.35;
			else if (event.key.keysym.sym == SDLK_END)
				light[1]-=0.35;
			else if (event.key.keysym.sym == SDLK_PAGEUP)
				light[2]+=0.35;
			else if (event.key.keysym.sym == SDLK_PAGEDOWN)
				light[2]-=0.35;
			break;
		}
	return temp;
}

int main(void)
{
	int finished, file, size, cam_loc, light_loc;
	float camera[3*4] = {
		0.,10., 0., 0.,
		0., 0., 0., 0.,
		1., 0., 0., 0.};

	file = open(FRAGMENT, O_RDONLY);
	size = lseek(file, 0, SEEK_END);
	lseek(file, 0, SEEK_SET);
	fragment = malloc(size);
	read(file, fragment, size);
	close(file);

	file = open(VECTOR, O_RDONLY);
	size = lseek(file, 0, SEEK_END);
	lseek(file, 0, SEEK_SET);
	vector = malloc(size);
	read(file, vector, size);
	close(file);

	screen = MakeGLWindow(XRES, YRES, 32);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
	glViewport(0, 0, screen->w, screen->h);

	SDL_Surface *img = SDL_LoadBMP("./texture.bmp");
	glEnable(GL_TEXTURE_2D);
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, img->w, img->h, 0,
		     GL_BGR, GL_UNSIGNED_BYTE, img->pixels);
	SDL_FreeSurface(img);

	compile_program(&program, (const char *)vector, (const char *)fragment);
	glUseProgram(program);
	cam_loc = glGetUniformLocation(program, "camera");
	light_loc = glGetUniformLocation(program, "l_loc");
	glUniform4fv(cam_loc, 3, camera);
	glUniform3f(light_loc, light[0], light[1],light[2]);
	glRects(-1, -1, 1, 1);
	finished = 0;
	while (!finished) {
		finished = do_events();
		SDL_GL_SwapBuffers();
		SDL_Delay(200);
	}

	cleanup();
	return 0;
}
