varying vec3 icamera[3];
varying vec3 campos;
varying vec3 raydir;
uniform vec4 camera[3];
uniform vec3 l_loc;

// lighting
//vec3 l_loc = vec3(-5.0, 8.0, -20.0);
//vec3 l_col = vec3(1.78, 1.576, 1.086);
//vec3 l_col = vec3(2.0, 1.843, 0.0);
vec3 l_col = vec3(0.5, 0.5, 0.5);

// material diffuse
vec3 m_dif = vec3(0.3, 0.3, 0.4);

// easy pseudo random variation because
// fucking nvidia cards fail the noise* functions
float rand2(vec2 xy)
{
	return fract(sin(dot(xy.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

// smooth cosine interpolation.
float interpolate(in float a, in float b, in float x)
{
	float f;
	f = (1.0 - cos(x*3.141592)) * 0.5;
	return ((a*(1.0-f)) + (b*f));
}

float smooth_noise(in vec2 xy)
{
	float corners, sides, center, x, y;
	x = xy.x;
	y = xy.y;
	corners = (rand2(vec2(x-1.0,y-1.0)) + rand2(vec2(x+1.0,y-1.0)) +
		   rand2(vec2(x-1.0,y+1.0)) + rand2(vec2(x+1.0,y+1.0))) * 0.0625;
	sides   = (rand2(vec2(x-1.0,y))     + rand2(vec2(x+1.0,y))     +
		   rand2(vec2(x,    y-1.0)) + rand2(vec2(x,    y+1.0))) * 0.125;
	center  = rand2(xy) * 0.25;

	return (corners + sides + center);
}

float interpolated_noise(in vec2 xy)
{
	float x,y,v1,v2,v3,v4,i1,i2;
	x = floor(xy.x);
	y = floor(xy.y);

	v1 = smooth_noise(vec2(x	,y	));
	v2 = smooth_noise(vec2(x+1.0 	,y	));
	v3 = smooth_noise(vec2(x	,y+1.0	));
	v4 = smooth_noise(vec2(x+1.0	,y+1.0	));

	i1 = interpolate(v1,v2, fract(xy.x));
	i2 = interpolate(v3,v4, fract(xy.x));

	return interpolate(i1,i2,fract(xy.y));
}

// Jan's variation of Perlin Noise 
float janperlin(in vec2 xy, in float dens, in float cov)
{
	float total = 0.0;
	float freq  = 1.00;
	float pers  = 0.46;
	float ampl  = 2.0;
	int octaves = 3;
	int i;

	for (i=0; i < octaves; i++) {
		total = total + interpolated_noise(vec2(xy.x * freq, xy.y * freq)) * ampl;
		freq  = freq * 2.0;
		ampl  = ampl * pers;
	}
	
	total = (total + cov) * dens;

	return total;
}

float calc_terrain_height(in vec2 xz)
{
	float t,d;

	// terrain
	t = janperlin(xz, 1.0, 0.0);

	// lake
	xz.y += 7.7;
	d = 16.0-xz.x*xz.x-xz.y*xz.y;
	if (d > 0.0) 
		d = 1.0 - sqrt(d)/6.0;
	else
		d = 1.0;

	// lake within terrain
	return t*d;
}

vec3 calc_norm(in vec3 p)
{
	vec3 n;
	n = vec3( calc_terrain_height(vec2(p.x-0.2,p.z)) - calc_terrain_height(vec2(p.x+0.2,p.z)),
		  0.4,
		  calc_terrain_height(vec2(p.x,p.z-0.2)) - calc_terrain_height(vec2(p.x,p.z+0.2)));
	return normalize(n);
}

vec4 calc_terrain_color(in vec3 l)
{
	vec4 color;
	float var;

	if (l.y > (1.9)) {
		color = (l.y * 0.5) * vec4(1.0, 1.0, 1.0, 0.0);
	} else if (l.y > (1.0)) {
		color = (2.0 * l.y) * vec4(0.28, 0.18, 0.06, 0.0);
	} else {
		color = (l.y * 0.83) * vec4(0.0, 0.1, 1.0, 0.0);
	}
	return color;
}

vec4 calc_lightning(in vec3 location, in vec3 nraydir)
{
	vec3 L, N, refl;
	float diff, sp;
	vec4 color;
	color = vec4(0.0, 0.0, 0.0, 0.0);
	
	L = l_loc;
	L = L - location;
	L = normalize(L);
	
	N = calc_norm(location);

	diff = dot(N, L);
	if (diff > 0.0001 ) {	
		color = diff * vec4(l_col.xzy,0.0);
		color = color * calc_terrain_color(location);
	}
	return color;
}

bool in_moon(in vec3 p)
{
	vec3 moon = vec3(10., 7., 1.);
	bool within = false;
	float u,v,t;

	u = (p.x - moon.x);
	v = (p.y - moon.y);
	t = u*u + v*v - (moon.z*moon.z);

	within = (t < 0.0);

	return within;	
}

vec4 sky(in vec3 p)
{
	float cloud=0.0;
	float moon;

	// cloud layer
//	if ((p.y < 9.0) && (p.y > 6.0)) {
		cloud = janperlin(p.xy, 0.23, 0.3);
//		cloud = cloud * 0.5 * sin( (p.y-6.1));
//	}
	
	if (in_moon(p)) 
		cloud += 0.3 * janperlin(p.yz, 0.2, 0.2);
	if (cloud < 0.45) cloud = 0.0;
	else cloud = cloud * p.y / 8.0;
/*
	} else {
		cloud = 0.01 * sin((20.0-p.y)*0.05) + cloud;
	}
*/	
	return vec4(cloud, cloud, cloud, 0.0);
}

vec4 cast_ray(in vec3 cam, in vec3 nraydir)
{
	vec3 p;
	float delt = 0.01;
	float mint = 0.001;
	float maxt = 30.0;
	float lh = 0.0;
	float ly = 0.0;
	float t;
	float h;

	for (t = mint; t < maxt; t += delt) {
		p = cam + nraydir * t;
		h = calc_terrain_height(p.xz);
		if ( p.y < h ) {
			// return terrain like colour.
			return calc_lightning(p, nraydir);
		}
		delt = 0.01 * t;
		lh = h;
		ly = p.y;
	}
	// return sky
	return sky(p);
}

void main(void)
{
	vec3 cam = campos.xyz;
	vec3 nraydir = normalize(raydir);

	gl_FragColor = cast_ray(cam, nraydir);
};

